@@ -1,10 +1,14 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena and contributors <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Fri Jun 28 19:23:52 2019
-
-@author: yoelr
 """
 import thermosteam as tmo
+import flexsolve as flx
 from . import _parse as prs
 from ..utils import chemicals_user
 from .._phase import NoPhase
@@ -14,6 +18,13 @@ import numpy as np
 
 __all__ = ('Reaction', 'ParallelReaction', 'SeriesReaction')
 
+def react_stream_adiabatically(stream, reaction):
+    if not isinstance(stream, tmo.Stream):
+            raise ValueError(f"stream must be a Stream object, not a '{type(stream).__name__}' object")
+    Hnet = stream.Hnet
+    reaction(stream)
+    stream.H = Hnet - stream.Hf
+
 def check_material_feasibility(material: np.ndarray):
     if (material < 0.).any(): raise InfeasibleRegion('not enough reactants; reaction conversion')
 
@@ -69,8 +80,11 @@ class Reaction:
     
     Other Parameters
     ----------------
-    check_material_balance=False: bool
+    check_mass_balance=False: bool
         Whether to assert that mass is not created or destroyed.
+    correct_mass_balance=False: bool
+        Whether to make sure mass is not created or destroyed by varying the 
+        reactant stoichiometric coefficient.
     check_atomic_balance=False: bool
         Whether to assert that stoichiometric balance by atoms cancel out.
     correct_atomic_balance=False: bool
@@ -148,9 +162,10 @@ class Reaction:
     
     def __init__(self, reaction, reactant, X, 
                  chemicals=None, basis='mol', *,
-                 check_material_balance=False,
+                 check_mass_balance=False,
                  check_atomic_balance=False,
-                 correct_atomic_balance=False):
+                 correct_atomic_balance=False,
+                 correct_mass_balance=False):
         if basis in ('wt', 'mol'):
             self._basis = basis
         else:
@@ -164,10 +179,12 @@ class Reaction:
             if correct_atomic_balance:
                 self.correct_atomic_balance()
             else:
+                if correct_mass_balance:
+                    self.correct_mass_balance()
+                elif check_mass_balance:
+                    self.check_mass_balance()
                 if check_atomic_balance:
                     self.check_atomic_balance()
-                elif check_material_balance:
-                    self.check_material_balance()
         else:
             self._stoichiometry = np.zeros(chemicals.size)
             self._X_index = self._chemicals.index(reactant)
@@ -272,6 +289,53 @@ class Reaction:
                                            self._chemicals)
         self._reaction(material_array)
     
+    def adiabatic_reaction(self, stream):
+        """
+        React stream material adiabatically, accounting for the change in enthalpy
+        due to the heat of reaction.
+        
+        Examples
+        --------
+        Note how the stream temperature changed after the reaction due to the heat of reaction:
+        
+        >>> import thermosteam as tmo
+        >>> import thermosteam.reaction as rxn
+        >>> chemicals = tmo.Chemicals(['H2', 'O2', 'H2O'])
+        >>> tmo.settings.set_thermo(chemicals)
+        >>> reaction = rxn.Reaction('2H2 + O2 -> 2H2O', reactant='H2', X=0.7)
+        >>> s1 = tmo.Stream('s1', H2=10, O2=20, H2O=1000)
+        >>> s2 = tmo.Stream('s2')
+        >>> s2.copy_like(s1) # s1 and s2 are the same
+        >>> s1.show() # Before reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): H2   10
+                         O2   20
+                         H2O  1e+03
+        
+        >>> reaction.show()
+        Reaction (by mol):
+         stoichiometry       reactant    X[%]
+         H2 + 0.5 O2 -> H2O  H2         70.00
+        
+        >>> reaction(s1) 
+        >>> s1.show() # After non-adiabatic reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): H2   3
+                         O2   16.5
+                         H2O  1.01e+03
+        
+        >>> reaction.adiabatic_reaction(s2)
+        >>> s2.show() # After adiabatic reaction
+        Stream: s2
+         phase: 'l', T: 324.11 K, P: 101325 Pa
+         flow (kmol/hr): H2   3
+                         O2   16.5
+                         H2O  1.01e+03
+        """
+        react_stream_adiabatically(stream, self)
+    
     def _reaction(self, material_array):
         material_array += material_array[self._X_index] * self.X * self._stoichiometry
     
@@ -339,7 +403,7 @@ class Reaction:
             stoichiometry_by_mol = self._stoichiometry
         return stoichiometry_by_mol
     
-    def check_material_balance(self, tol=1e-3):
+    def check_mass_balance(self, tol=1e-3):
         """Assert that stoichiometric mass balance is correct."""
         stoichiometry_by_wt = self._get_stoichiometry_by_wt()
         error = abs(stoichiometry_by_wt.sum())
@@ -348,7 +412,7 @@ class Reaction:
         )
     
     def check_atomic_balance(self, tol=1e-3):
-        """Assert that stoichiometric mass balance is correct."""
+        """Assert that stoichiometric atomic balance is correct."""
         stoichiometry_by_mol = self._get_stoichiometry_by_mol()
         formula_array = self.chemicals.formula_array
         unbalanced_array = formula_array @ stoichiometry_by_mol
@@ -358,6 +422,26 @@ class Reaction:
             + "\n ".join([f"{symbol}: {value}" for symbol, value in atoms.items()])
         )
     
+    def correct_mass_balance(self, variable=None):
+        """
+        Make sure mass is not created or destroyed by varying the 
+        reactant stoichiometric coefficient.
+        """
+        if variable:
+            index = self.chemicals.get_index(variable)
+        else:
+            index = self._X_index
+        stoichiometry_by_wt = self._get_stoichiometry_by_wt()
+        
+        def f(x):
+            stoichiometry_by_wt[index] = x
+            return stoichiometry_by_wt.sum()
+        
+        flx.aitken_secant(f, 1)
+        if self._basis == 'mol': 
+            self._stoichiometry[:] = stoichiometry_by_wt / self.MWs
+        self._rescale()
+    
     def correct_atomic_balance(self, constants=None):
         """Correct stoichiometry coffecients to satisfy atomic balance."""
         stoichiometry_by_mol = self._get_stoichiometry_by_mol()
@@ -379,8 +463,7 @@ class Reaction:
             raise RuntimeError(
                  'to solve atomic balance, number of atoms '
                 f'({M_atoms} available) must be equal to the number of '
-                 'chemicals, not including the reactant '
-                f'({N_chemicals} available)'
+                f'varied chemicals ({N_chemicals} available)'
             )
         x = np.linalg.solve(A, b)
         stoichiometry_by_mol[chemical_index] = x.flatten()
@@ -593,6 +676,63 @@ class ParallelReaction(ReactionSet):
         """React material ignoring feasibility checks."""
         material_array = as_material_array(material, self._basis, self._chemicals)
         self._reaction(material_array)
+        
+    def adiabatic_reaction(self, stream):
+        """
+        React stream material adiabatically, accounting for the change in enthalpy
+        due to the heat of reaction.
+        
+        Examples
+        --------
+        Note how the stream temperature changed after the reaction due to the heat of reaction:
+            
+        >>> import thermosteam as tmo
+        >>> import thermosteam.reaction as rxn
+        >>> chemicals = tmo.Chemicals(['H2', 'CH4', 'O2', 'CO2', 'H2O'])
+        >>> tmo.settings.set_thermo(chemicals)
+        >>> reaction = rxn.ParallelReaction([
+        ...    #            Reaction definition          Reactant    Conversion
+        ...    rxn.Reaction('2H2 + O2 -> 2H2O',        reactant='H2',  X=0.7),
+        ...    rxn.Reaction('CH4 + O2 -> CO2 + 2H2O',  reactant='CH4', X=0.1)
+        ...    ])
+        >>> s1 = tmo.Stream('s1', H2=10, CH4=5, O2=100, H2O=1000)
+        >>> s2 = tmo.Stream('s2')
+        >>> s2.copy_like(s1) # s1 and s2 are the same
+        >>> s1.show() # Before reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): H2   10
+                         CH4  5
+                         O2   100
+                         H2O  1e+03
+        
+        >>> reaction.show()
+        ParallelReaction (by mol):
+        index  stoichiometry            reactant    X[%]
+        [0]    H2 + 0.5 O2 -> H2O       H2         70.00
+        [1]    CH4 + O2 -> CO2 + 2 H2O  CH4        10.00
+        
+        >>> reaction(s1)
+        >>> s1.show() # After non-adiabatic reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): H2   3
+                         CH4  4.5
+                         O2   96
+                         CO2  0.5
+                         H2O  1.01e+03
+        
+        >>> reaction.adiabatic_reaction(s2)
+        >>> s2.show() # After adiabatic reaction
+        Stream: s2
+         phase: 'l', T: 328.96 K, P: 101325 Pa
+         flow (kmol/hr): H2   3
+                         CH4  4.5
+                         O2   96
+                         CO2  0.5
+                         H2O  1.01e+03
+        """
+        react_stream_adiabatically(stream, self)
 
     def _reaction(self, material_array):
         material_array += material_array[self._X_index] * self.X @ self._stoichiometry
@@ -650,6 +790,62 @@ class SeriesReaction(ReactionSet):
         """React material ignoring feasibility checks."""
         array = as_material_array(material, self._basis, self._chemicals)
         self._reaction(array)
+        
+    def adiabatic_reaction(self, stream):
+        """
+        React stream material adiabatically, accounting for the change in enthalpy
+        due to the heat of reaction.
+        
+        Examples
+        --------
+        Note how the stream temperature changed after the reaction due to the heat of reaction:
+        
+        >>> import thermosteam as tmo
+        >>> import thermosteam.reaction as rxn
+        >>> chemicals = tmo.Chemicals(['CH4', 'CO','O2', 'CO2', 'H2O'])
+        >>> tmo.settings.set_thermo(chemicals)
+        >>> reaction = rxn.SeriesReaction([
+        ...     #            Reaction definition                 Reactant       Conversion
+        ...     rxn.Reaction('2CH4 + 3O2 -> 2CO + 4H2O',       reactant='CH4',    X=0.7),
+        ...     rxn.Reaction('2CO + O2 -> 2CO2',               reactant='CO',     X=0.1)
+        ...     ])
+        >>> s1 = tmo.Stream('s1', CH4=5, O2=100, H2O=1000)
+        >>> s2 = tmo.Stream('s2')
+        >>> s2.copy_like(s1) # s1 and s2 are the same
+        >>> s1.show() # Before reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): CH4  5
+                         O2   100
+                         H2O  1e+03
+        
+        >>> reaction.show()
+        SeriesReaction (by mol):
+        index  stoichiometry               reactant    X[%]
+        [0]    CH4 + 1.5 O2 -> CO + 2 H2O  CH4        70.00
+        [1]    CO + 0.5 O2 -> CO2          CO         10.00
+        
+        >>> reaction(s1)
+        >>> s1.show() # After non-adiabatic reaction
+        Stream: s1
+         phase: 'l', T: 298.15 K, P: 101325 Pa
+         flow (kmol/hr): CH4  1.5
+                         CO   3.15
+                         O2   94.6
+                         CO2  0.35
+                         H2O  1.01e+03
+        
+        >>> reaction.adiabatic_reaction(s2)
+        >>> s2.show() # After adiabatic reaction
+        Stream: s2
+         phase: 'l', T: 326.12 K, P: 101325 Pa
+         flow (kmol/hr): CH4  1.5
+                         CO   3.15
+                         O2   94.6
+                         CO2  0.35
+                         H2O  1.01e+03
+        """
+        react_stream_adiabatically(stream, self)
 
     def reduce(self):
         raise TypeError('cannot reduce a SeriesReation object, only '
@@ -678,4 +874,3 @@ class SeriesReaction(ReactionSet):
 # RxnS = ReactionSet
 # PRxn = ParallelReaction
 # SRxn = SeriesReaction
-
